package cn.com.taiji.cql.service.impl;

import cn.com.taiji.common.util.HashCodeUtil;
import cn.com.taiji.cql.service.IBuildDataStore;
import lombok.extern.slf4j.Slf4j;
import org.geotools.api.data.DataStore;
import org.geotools.data.jdbc.datasource.DBCPDataSourceFactory;
import org.geotools.data.jdbc.datasource.DataSourceFinder;
import org.geotools.data.mysql.MySQLDataStoreFactory;
import org.geotools.jdbc.JDBCDataStoreFactory;
import org.springframework.stereotype.Service;

import javax.sql.DataSource;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

@Slf4j
@Service
public class BuildDataStoreImpl implements IBuildDataStore {

    private Map<String,DataStore> dataStoreMaps = new ConcurrentHashMap<String,DataStore>();


    @Override
    public DataStore createDataStore(Map<String, Object> mapParams) throws Exception {

        String host = mapParams.get(JDBCDataStoreFactory.HOST.key).toString();

        String port = mapParams.get(JDBCDataStoreFactory.PORT.key).toString();

        String database = mapParams.get(JDBCDataStoreFactory.DATABASE.key).toString();

        int hashKey = HashCodeUtil.hashCode(host+"-"+port+"-"+database);

        JDBCDataStoreFactory jDBCDataStoreFactory = null;

        DataStore dataStore = null;

        if(dataStoreMaps.containsKey(String.valueOf(hashKey))){

            dataStore = dataStoreMaps.get(String.valueOf(hashKey));

            log.info("系统从内存连接到位于：" + host + "的空间数据库" + database + "成功！");

        }else{


            DataSource dataSource = DataSourceFinder.getDataSource(mapParams);

            if(dataSource !=null){

                if(mapParams.get(DBCPDataSourceFactory.DRIVERCLASS.key).equals("com.mysql.cj.jdbc.Driver")){

                    jDBCDataStoreFactory = new MySQLDataStoreFactory();
                }

                Map<String, Object> dataStoreParams = new HashMap<String, Object>();

                dataStoreParams.put(JDBCDataStoreFactory.SCHEMA.key, database);
                dataStoreParams.put(JDBCDataStoreFactory.DATASOURCE.key,dataSource);
                dataStoreParams.put(JDBCDataStoreFactory.BATCH_INSERT_SIZE.key, 1000);

                dataStore = jDBCDataStoreFactory.createDataStore(dataStoreParams);

                dataStoreMaps.put(String.valueOf(hashKey),dataStore);

                log.info("系统新建连接到位于：" + host + "的空间数据库" + database + "成功！");

            }else{

                log.info("获取"+host+"-"+port+"-"+database+"的数据源连接池失败！");
            }



        }

        return dataStore;

    }
}
