package cn.com.taiji.tile.model;

import lombok.Data;

@Data
public class Point {
    private double x;
    private double y;
    private double z;

    public Point(double x, double y, boolean round) {
        this.x = round ? Math.round(x) : x;
        this.y = round ? Math.round(y) : y;
    }

    public Point(double x, double y) {
        this(x, y, false);
    }

    public Point(double x, double y, double z) {
        this(x, y, false);
        this.z = z;
    }

    public Point(Point p) {
        this(p.x, p.y);
    }


    public Point(int[] p) {
        this(p[0], p[1]);
    }


    public Point clone() {
        return new Point(this.x, this.y);
    }

    public Point add(Point point) {
        return clone()._add(new Point(point));
    }


    private Point _add(Point point) {
        this.x += point.x;
        this.y += point.y;
        return this;
    }

    public Point subtract(Point point) {
        return clone()._subtract(point);
    }

    Point _subtract(Point point) {
        this.x -= point.x;
        this.y -= point.y;
        return this;
    }

    public Point divideBy(double num) {
        return clone()._divideBy(num);
    }

    private Point _divideBy(double num) {
        this.x /= num;
        this.y /= num;
        return this;
    }

    public Point multiplyBy(double num) {
        return clone()._multiplyBy(num);
    }

    private Point _multiplyBy(double num) {
        this.x *= num;
        this.y *= num;
        return this;
    }

    public Point scaleBy(Point point) {
        return new Point(this.x * point.x, this.y * point.y);
    }

    public Point unscaleBy(Point point) {
        return new Point(this.x / point.x, this.y / point.y);
    }

    public Point round() {
        return clone()._round();
    }

    Point _round() {
        this.x = Math.round(this.x);
        this.y = Math.round(this.y);
        return this;
    }

    public Point floor() {
        return clone()._floor();
    }

    private Point _floor() {
        this.x = Math.floor(this.x);
        this.y = Math.floor(this.y);
        return this;
    }


    public Point ceil() {
        return clone()._ceil();
    }

    private Point _ceil() {
        this.x = Math.ceil(this.x);
        this.y = Math.ceil(this.y);
        return this;
    }


    public double distanceTo(Point point) {
        double x = point.x - this.x;
        double y = point.y - this.y;
        return Math.sqrt(x * x + y * y);
    }

    public boolean contains(Point point) {
        return (Math.abs(point.x) <= Math.abs(this.x) && Math.abs(point.y) <= Math.abs(this.y));
    }
}
