package cn.com.taiji.tile.service.impl;

import cn.com.taiji.common.domain.PageResult;
import cn.com.taiji.common.model.*;
import cn.com.taiji.cql.service.IECqlService;
import cn.com.taiji.cql.service.ISqlService;
import cn.com.taiji.tile.model.FeatureInfo;
import cn.com.taiji.tile.model.POI;
import cn.com.taiji.tile.service.ITileService;
import cn.com.taiji.tile.util.ShapeUtils;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.geotools.api.filter.Filter;
import org.geotools.data.jdbc.datasource.DBCPDataSourceFactory;
import org.geotools.data.mysql.MySQLFilterToSQL;
import org.geotools.filter.text.ecql.ECQL;
import org.geotools.jdbc.JDBCDataStoreFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.imageio.ImageIO;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.CopyOnWriteArrayList;

@Slf4j
@Service
public class TileServiceImpl implements ITileService {

    @Autowired
    private IECqlService ecqlService;

    @Autowired
    private ISqlService sqlService;


    @Override
    public byte[] tileData(GeoRequestParam requestParam, List<LayerGroupDsView> layerGroupDsViewList, Map<String, List<LayerStyleView>> dataMaps) {
        ConcurrentHashMap<String, BufferedImage> biMap = new ConcurrentHashMap<>();
        layerGroupDsViewList.parallelStream().forEach(layerGroupDsView -> {
            Result result = getResult(requestParam, dataMaps, layerGroupDsView);
            if (result.pageResult != null && result.pageResult.getRecordCount() > 0) {
                List<Map<String, Object>> cqlDataList = (List<Map<String, Object>>) result.pageResult.getDatas();
                //3根据空间地理数据生成图层二进制数据
                BufferedImage bufferedImage = ShapeUtils.drawImage(requestParam.getBbox(), cqlDataList, layerGroupDsView.getBoxColumn(), result.layerStyleViewList);
                biMap.put(layerGroupDsView.getLayerCode(),bufferedImage);
            } else {
//                log.info("根据查询过滤规则条件:" + filterCqlStr.toString() + "和图层代码:" + requestParam.getLayers() + " 没有找到图层对应的数据源的空间地理数据");
            }
        });
        BufferedImage image = new BufferedImage(requestParam.getWidth(), requestParam.getHeight(), BufferedImage.TYPE_INT_ARGB);
        Graphics2D g2 = image.createGraphics();
        layerGroupDsViewList.forEach(item-> g2.drawImage(biMap.get(item.getLayerCode()), 0, 0, null));
        ByteArrayOutputStream buffer = new ByteArrayOutputStream();
        try {
            ImageIO.write(image, "png", buffer);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return buffer.toByteArray();
    }

    private Result getResult(GeoRequestParam requestParam, Map<String, List<LayerStyleView>> dataMaps, LayerGroupDsView layerGroupDsView) {
        //获取图层代码对应的样式
        List<LayerStyleView> layerStyleViewList = new ArrayList<>();
        if (dataMaps.containsKey(layerGroupDsView.getLayerCode())) {
            layerStyleViewList = dataMaps.get(layerGroupDsView.getLayerCode());
        }
        String host = layerGroupDsView.getHost().split(",")[0].split(":")[0];
        String port = layerGroupDsView.getHost().split(",")[0].split(":")[1];

        Map<String, Object> cqlDsParams = new HashMap<>();
        cqlDsParams.put(DBCPDataSourceFactory.DSTYPE.key, "DBCP");
        cqlDsParams.put(DBCPDataSourceFactory.DRIVERCLASS.key, "com.mysql.cj.jdbc.Driver");
        cqlDsParams.put(DBCPDataSourceFactory.JDBC_URL.key, "jdbc:mysql://" + host + ":" + port + "/" + layerGroupDsView.getDataBase() + "?useUnicode=true&characterEncoding=UTF-8&autoReconnect=true&useSSL=false&zeroDateTimeBehavior=convertToNull&serverTimezone=Asia/Shanghai");
        cqlDsParams.put(DBCPDataSourceFactory.USERNAME.key, layerGroupDsView.getUserName());
        cqlDsParams.put(DBCPDataSourceFactory.PASSWORD.key, layerGroupDsView.getUserPassword());
        cqlDsParams.put(DBCPDataSourceFactory.MAXACTIVE.key, Integer.valueOf(10));
        cqlDsParams.put(DBCPDataSourceFactory.MAXIDLE.key, Integer.valueOf(1));
        cqlDsParams.put(JDBCDataStoreFactory.HOST.key, host);
        cqlDsParams.put(JDBCDataStoreFactory.DATABASE.key, layerGroupDsView.getDataBase());
        cqlDsParams.put(JDBCDataStoreFactory.PORT.key, port);
        cqlDsParams.put("tableName", layerGroupDsView.getTableName());
        String layerDataType = layerGroupDsView.getLayerDataType();
        String layerDataRun = layerGroupDsView.getLayerDataRun();
        PageResult pageResult = null;
        StringBuilder filterCqlStr = new StringBuilder();
        if (StringUtils.equals(layerDataType,"cql")) {
            //2根据cql规则获取图层对应的空间地理数据
            String filterCql = requestParam.getCqlFilter();
            if (StringUtils.isNotBlank(filterCql)) {
                filterCqlStr.append(filterCql);
            }
            String bbox = requestParam.getBbox();
            if (StringUtils.isNotBlank(bbox)) {
                StringBuilder sbBbox = new StringBuilder();
                sbBbox.append("(bbox (");
                sbBbox.append("location,");
                sbBbox.append(requestParam.getBbox()).append("))");
                if (filterCqlStr.length() > 0) {

                    filterCqlStr.append(" and " + sbBbox.toString());
                } else {

                    filterCqlStr.append(sbBbox.toString());

                }

            }
            if (layerDataRun != null && !"".equals(layerDataRun)) {
                if (filterCqlStr.length() > 0) {
                    filterCqlStr.append(" and " + layerDataRun);
                } else {
                    filterCqlStr.append(layerDataRun);
                }
            }
            System.out.println("查询条件:" + filterCqlStr.toString());
            Map<String, Object> mapParams = new HashMap<String, Object>();
            if (filterCqlStr.length() > 0) {
                mapParams.put("cql", filterCqlStr.toString());
            }
            pageResult = ecqlService.cqlQuery(cqlDsParams, mapParams);
        } else if (layerDataType != null && !"".equals(layerDataType) && "sql".equals(layerDataType)) {

            if (layerDataRun != null && !"".equals(layerDataRun)) {

                Map<String,Object> viewParamMaps = new HashMap<String,Object>();
                String viewParams = requestParam.getViewparams();
                if(viewParams !=null && !"".equals(viewParams)){

                    String[] viewParamArr = viewParams.split(";");
                    for(String viewData:viewParamArr){

                        String[] viewValue = viewData.split(":");
                        if(viewValue.length >1){
                            viewParamMaps.put(viewValue[0],viewValue[1]);
                        }


                    }

                }

                Map<String,SqlJsonData> sqlParamMaps = new HashMap<String,SqlJsonData>();

                String sqlJson = layerGroupDsView.getJson();

                if(sqlJson !=null && !"".equals(sqlJson)){

                    List<SqlJsonData> sqlParamList = JSONObject.parseArray(sqlJson, SqlJsonData.class);

                    for(SqlJsonData sqlJsonData:sqlParamList){

                        if(viewParamMaps.containsKey(sqlJsonData.getName())){

                            sqlJsonData.setData((String) viewParamMaps.get(sqlJsonData.getName()));

                        }

                        sqlParamMaps.put(sqlJsonData.getName(),sqlJsonData);
                    }
                }

                for(String paramKey:sqlParamMaps.keySet()){

                    String paramData = sqlParamMaps.get(paramKey).getData();

                    if(paramData == null || "".equals(paramData)){

                        paramData = sqlParamMaps.get(paramKey).getValue();

                    }

                    layerDataRun = layerDataRun.replace("%" + paramKey + "%", paramData);

                }

                String cqlFilter = requestParam.getCqlFilter();

                if(cqlFilter !=null && !"".equals(cqlFilter)){

                    try{

                        MySQLFilterToSQL filterToSql = new MySQLFilterToSQL(true);
                        filterToSql.setInline(true);

                        Filter filter = ECQL.toFilter(cqlFilter);

                        String actual = filterToSql.encodeToString(filter);

                        if(actual.indexOf("))', null))") >0){

                            actual = actual.replace("))', null))","))'))");

                        }

                        layerDataRun = layerDataRun + " and " +actual;

                    }catch (Exception exception){
                        exception.printStackTrace();
                    }



                }

                Object[] bindArgs = new Object[0];

                pageResult = sqlService.sqlQuery4Jdbc(cqlDsParams, layerDataRun, bindArgs);



            } else {

                log.info("执行图层数据类型:" + layerDataType + "和图层代码:" + layerGroupDsView.getLayerCode() + " 的执行内容为空或者为null");
            }
        }
        Result result = new Result(layerStyleViewList, pageResult);
        return result;
    }



    public static List<POI> convert(List<Map<String, Object>> maps,String column) {
        List<POI> pois = new ArrayList<>();
        for (Map<String, Object> map : maps) {

            POI poi = new POI();
            poi.setShape(map.get(column).toString());
            pois.add(poi);
        }
        return pois;
}


    @Override
    public FeatureInfo getFeatureInfo(GeoRequestParam requestParam, List<LayerGroupDsView> layerGroupDsViewList, Map<String, List<LayerStyleView>> dataMaps) {
        FeatureInfo featureInfo = new FeatureInfo();
        List<Map<String, Object>> features = new CopyOnWriteArrayList<>();
        layerGroupDsViewList.parallelStream().forEach(layerGroupDsView -> {
            Result result = getResult(requestParam, dataMaps, layerGroupDsView);
            if (result.pageResult != null && result.pageResult.getRecordCount() > 0) {
                List<Map<String, Object>> cqlDataList = (List<Map<String, Object>>) result.pageResult.getDatas();
                //3根据空间地理数据生成图层二进制数据
                List<Map<String, Object>> maps = ShapeUtils.detectClick(requestParam.getBbox(), cqlDataList, layerGroupDsView.getBoxColumn(), result.layerStyleViewList, requestParam.getX(), requestParam.getY());
                features.addAll(maps);
            } else {
//                log.info("根据查询过滤规则条件:" + filterCqlStr.toString() + "和图层代码:" + requestParam.getLayers() + " 没有找到图层对应的数据源的空间地理数据");
            }
        });
        featureInfo.setFeatures(features);
        featureInfo.setNumberReturned(features.size());
        featureInfo.setTimeStamp(DateUtil.format(new Date(),DateTimeFormatter.ISO_INSTANT));
        return featureInfo;
    }

    @Override
    public List<Map<String, Object>> getFeature(GeoRequestParam requestParam, List<LayerGroupDsView> layerGroupDsViewList, Map<String, List<LayerStyleView>> dataMaps) {
        List<Map<String, Object>> list = new CopyOnWriteArrayList<>();
        layerGroupDsViewList.parallelStream().forEach(layerGroupDsView -> {
            Result result = getResult(requestParam, dataMaps, layerGroupDsView);
            if (result.pageResult != null && result.pageResult.getRecordCount() > 0) {
                List<Map<String, Object>> cqlDataList = (List<Map<String, Object>>) result.pageResult.getDatas();
                list.addAll(cqlDataList);
            } else {
//                log.info("根据查询过滤规则条件:" + filterCqlStr.toString() + "和图层代码:" + requestParam.getLayers() + " 没有找到图层对应的数据源的空间地理数据");
            }
        });
        return list;
    }

    public static double[] parseBbox(String bboxString) {
        String[] bboxValuesStr = bboxString.split(",");
        if (bboxValuesStr.length != 4) {
            return new double[0]; // 返回空数组表示无效的 BBOX 格式
        }

        double[] bboxValues = new double[4];
        try {
            for (int i = 0; i < 4; i++) {
                bboxValues[i] = Double.parseDouble(bboxValuesStr[i]);
            }
        } catch (NumberFormatException e) {
            return new double[0]; // 返回空数组表示无效的 BBOX 格式
        }

        return bboxValues;
    }

    private static class Result {
        public final List<LayerStyleView> layerStyleViewList;
        public final PageResult pageResult;

        public Result(List<LayerStyleView> layerStyleViewList, PageResult pageResult) {
            this.layerStyleViewList = layerStyleViewList;
            this.pageResult = pageResult;
        }
    }
}
