package cn.com.taiji.tile.util;

import java.awt.*;
import java.awt.image.*;
import java.util.Map;
import java.util.TreeMap;


public class HotImageTileUtils {
    private static final Color rbg0_5 = new Color(0, 180, 0, 20);
    private static final Color rbg5_10 = new Color(0, 200, 0, 40);
    private static final Color rbg10_25 = new Color(50, 255, 0, 80);
    private static final Color rbg25_50 = new Color(0, 255, 0, 160);
    private static final Color rbg50_100 = new Color(255, 255, 0, 185);
    private static final Color rbg100_125 = new Color(255, 213, 0, 200);
    private static final Color rbg125_150 = new Color(255, 171, 0, 215);
    private static final Color rbg125_150_2 = new Color(255, 129, 0, 225);
    private static final Color rbg150_175 = new Color(255, 87, 0, 235);
    private static final Color rbg175_200 = new Color(255, 42, 0, 245);
    private static final Color rbg200_ = new Color(255, 0, 0, 255);
    private static final RGBTable pressures = new RGBTable(64);
    private static final RGBTable pressuresColor = new RGBTable(5);
    private static final TreeMap<Double, Color> colors = new TreeMap<>();

    static {
        colors.put(Double.valueOf("0.0D"), new Color(0, 0, 0, 0));
        colors.put(Double.valueOf("0.1D"), rbg0_5);
        colors.put(Double.valueOf("3.0D"), rbg5_10);
        colors.put(Double.valueOf("15.0D"), rbg10_25);
        colors.put(Double.valueOf("25.0D"), rbg25_50);
        colors.put(Double.valueOf("35.0D"), rbg50_100);
        colors.put(Double.valueOf("55.0D"), rbg100_125);
        colors.put(Double.valueOf("85.0D"), rbg125_150);
        colors.put(Double.valueOf("125.0D"), rbg125_150_2);
        colors.put(Double.valueOf("135.0D"), rbg150_175);
        colors.put(Double.valueOf("145.0D"), rbg175_200);
        colors.put(Double.valueOf("150.0D"), rbg200_);
    }

    public static boolean gradual = false;

    public static void press(double[] matrix, int px, int py, int w) {
        for (int dx = -64; dx <= 64; dx++) {
            for (int dy = -64; dy <= 64; dy++) {
                int x = px + dx, y = py + dy;
                if (0 <= x && x < 256 && 0 <= y && y < 256) {
                    matrix[x * 256 + y] = matrix[x * 256 + y] + w * pressures.getPressure(dx, dy);
                }
            }
        }
    }


    public static Color toRGB(double presure) {
        Map.Entry<Double, Color> floor = colors.floorEntry(Double.valueOf(presure));
        Map.Entry<Double, Color> ceiling = colors.ceilingEntry(Double.valueOf(presure));
        if (floor == null) {
            return ceiling.getValue();
        }
        if (ceiling == null) {
            return floor.getValue();
        }
        if (floor.getKey().equals(ceiling.getKey())) {
            return floor.getValue();
        }
        float factor = (float) ((presure - ((Double) floor.getKey()).doubleValue()) / (((Double) ceiling.getKey()).doubleValue() - ((Double) floor.getKey()).doubleValue()));
        return mix(factor, floor.getValue(), ceiling.getValue());
    }


    private static Color mix(float factor, Color color1, Color color2) {
        int r = (int) (color1.getRed() * (1.0F - factor) + color2.getRed() * factor);
        int g = (int) (color1.getGreen() * (1.0F - factor) + color2.getGreen() * factor);
        int b = (int) (color1.getBlue() * (1.0F - factor) + color2.getBlue() * factor);
        int a = (int) (color1.getAlpha() * (1.0F - factor) + color2.getAlpha() * factor);
        return new Color(r, g, b, a);
    }


    private static BufferedImage blur(BufferedImage source, int radius) {
        BufferedImage img = new BufferedImage(source.getWidth() + radius * 2, source.getHeight() + radius * 2, 3);
        Graphics2D g2 = (Graphics2D) img.getGraphics();
        g2.setColor(new Color(0, 0, 0, 0));
        g2.fillRect(0, 0, source.getWidth() + radius * 2, source.getHeight() + radius * 2);
        g2.drawImage(source, radius, radius, (ImageObserver) null);
        g2.dispose();
        int square = radius * radius;
        float sum = 0.0F;
        float[] matrix = new float[square];
        int i;
        for (i = 0; i < square; i++) {
            int dx = i % radius - radius / 2;
            int dy = i / radius - radius / 2;
            matrix[i] = (float) (radius - Math.sqrt((dx * dx + dy * dy)));
            sum += matrix[i];
        }
        for (i = 0; i < square; i++) {
            matrix[i] = matrix[i] / sum;
        }
        BufferedImageOp op = new ConvolveOp(new Kernel(radius, radius, matrix), 0, null);
        BufferedImage res = op.filter(img, null);
        BufferedImage out = new BufferedImage(source.getWidth(), source.getHeight(), 6);
        Graphics2D g3 = (Graphics2D) out.getGraphics();
        g3.drawImage(res, -radius, -radius, (ImageObserver) null);
        g3.dispose();
        return out;
    }

}