package cn.com.taiji.web.controller;



import cn.com.taiji.common.domain.Result;
import cn.com.taiji.common.model.LayerGroupDsView;
import cn.com.taiji.common.model.LayerStyleView;
import cn.com.taiji.common.model.GeoRequestParam;
import cn.com.taiji.tile.model.FeatureInfo;
import cn.com.taiji.tile.service.ITileService;
import cn.com.taiji.web.service.IGeoDataSourceService;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.PostConstruct;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.*;

/**
 * 图层通用查询控制器
 *
 * @author makejava
 * @since 2021-04-22 16:53:35
 */
@Slf4j
@RestController
@Api("图层通用查询控制器")
public class TileWebController {

    @Autowired
    private ITileService tileService;

    @Autowired
    private IGeoDataSourceService geoDataSourceService;


    @RequestMapping(value = {"/queryTile"}, method = {RequestMethod.GET, RequestMethod.POST})
    public void queryTile(HttpServletRequest request, HttpServletResponse response) {


        GeoRequestParam geoRequestParam = JSONObject.parseObject(JSONObject.toJSONString(getParameterMap(request)), GeoRequestParam.class);

        List<String> layerGroupList = new ArrayList<String>();

        //图层组
        String layers = geoRequestParam.getLayers();
        if(layers !=null && !"".equals(layers)){

            layerGroupList.add(layers);
        }
        //图层
        String typeName = geoRequestParam.getTypename();

        if(typeName !=null && !"".equals(typeName)){

            String[] typeNames = typeName.split(",");

            layerGroupList.addAll(Arrays.asList(typeNames));

        }

        if(layerGroupList.size() >0){

            try{

                List<LayerGroupDsView> layerGroupDsViewList = geoDataSourceService.layerCodeGeoDataSource(layerGroupList);

                if(layerGroupDsViewList !=null && layerGroupDsViewList.size()>0){

                    List<String> layerCodeList = new ArrayList<String>();

                    for(LayerGroupDsView layerGroupDsView:layerGroupDsViewList){

                        layerCodeList.add(layerGroupDsView.getLayerCode());
                    }

                    Map<String,List<LayerStyleView>> dataMaps = geoDataSourceService.layerCodeShapeStyle(layerCodeList);

                    if(StringUtils.equals(geoRequestParam.getRequest(),"GetMap")) {
                        byte[] ret = tileService.tileData(geoRequestParam, layerGroupDsViewList, dataMaps);
                        responseWrite(response, "image/png", ret);
                    }else if(StringUtils.equals(geoRequestParam.getRequest(),"GetFeatureInfo")) {
                        FeatureInfo featureInfo = tileService.getFeatureInfo(geoRequestParam, layerGroupDsViewList, dataMaps);
                        // 转换List为JSON
                        ObjectMapper mapper = new ObjectMapper();
                        String json = mapper.writeValueAsString(featureInfo);
                        // 设置响应类型和编码
                        response.setContentType("application/json");
                        response.setCharacterEncoding("UTF-8");
                        // 将JSON写入响应
                        response.getWriter().write(json);
                    } else if(StringUtils.equals(geoRequestParam.getRequest(),"GetFeature")){
                        List<Map<String, Object>> feature = tileService.getFeature(geoRequestParam, layerGroupDsViewList, dataMaps);
                        // 转换List为JSON
                        ObjectMapper mapper = new ObjectMapper();
                        String json = mapper.writeValueAsString(feature);
                        // 设置响应类型和编码
                        response.setContentType("application/json");
                        response.setCharacterEncoding("UTF-8");
                        // 将JSON写入响应
                        response.getWriter().write(json);
                    }
                }else{

                    log.info("根据图层代码：" + StringUtils.join(layerGroupList,",") + "获取图层对应的数据源为空或者为null");
                }

            }catch (Exception exception){

                exception.printStackTrace();

                log.error("根据图层代码：" + geoRequestParam.getLayers() +"获取数据源信息失败！"+exception);
            }



        }else{

            log.info("图层组layers参数为空或者图层typeName参数为空");
        }


    }

    private void responseWrite(HttpServletResponse response, String contentType, byte[] data) {

        response.setStatus(200);
        response.setContentType(contentType);

        if (data != null) {

            response.setContentLength(data.length);
            ServletOutputStream out = null;

            try {

                out = response.getOutputStream();
                out.write(data);
                out.flush();

            } catch (Exception e) {

                e.printStackTrace();

            } finally {

                if (out != null)
                    try {
                        out.close();
                    } catch (Exception e2) {

                        e2.printStackTrace();

                    }
            }
        }
    }

    /**
     * 将请求参数转化为Map
     *
     * @param request
     * @return
     */
    public static Map<String, Object> getParameterMap(HttpServletRequest request) {

        Map<String, Object> param = new HashMap<>();
        try {
            Enumeration<String> em = request.getParameterNames();
            while (em.hasMoreElements()) {
                String key = em.nextElement();
                param.put(key.toLowerCase(), request.getParameter(key));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return param;
    }



}
