package cn.com.taiji.web.service.impl;

import cn.com.taiji.common.domain.PageResult;
import cn.com.taiji.common.domain.Result;
import cn.com.taiji.cql.service.IECqlService;
import cn.com.taiji.cql.service.ISqlService;
import cn.com.taiji.web.config.EnterpriseConfig;
import cn.com.taiji.web.model.DeviceInfo;
import cn.com.taiji.web.model.EnterpriseInfo;
import cn.com.taiji.web.model.StaticsInfo;
import cn.com.taiji.web.service.IEnterpriseService;
import cn.com.taiji.web.vo.DeviceVo;
import cn.com.taiji.web.vo.EnterpriseVo;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.geotools.api.referencing.crs.CoordinateReferenceSystem;
import org.geotools.data.jdbc.datasource.DBCPDataSourceFactory;
import org.geotools.feature.visitor.CountVisitor;
import org.geotools.geometry.jts.JTS;
import org.geotools.jdbc.JDBCDataStoreFactory;
import org.geotools.referencing.crs.DefaultGeographicCRS;
import org.locationtech.jts.geom.Coordinate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;

@Slf4j
@Service
public class EnterpriseServiceImpl implements IEnterpriseService {

    @Autowired
    private EnterpriseConfig enterpriseConfig;

    @Autowired
    private IECqlService ecqlService;

    @Autowired
    private ISqlService sqlService;

    @Override
    public PageResult enterpriseQuery(EnterpriseVo enterpriseVo) {

        //数据源

        Map<String, Object> entDsParams = new HashMap<String, Object>();
        entDsParams.put(DBCPDataSourceFactory.DSTYPE.key, "DBCP");
        entDsParams.put(DBCPDataSourceFactory.DRIVERCLASS.key, "com.mysql.cj.jdbc.Driver");
        entDsParams.put(DBCPDataSourceFactory.JDBC_URL.key, "jdbc:mysql://"+enterpriseConfig.getDsIp()+":"+enterpriseConfig.getDsPort()+"/"+enterpriseConfig.getDsBaseName()+"?useUnicode=true&characterEncoding=UTF-8&autoReconnect=true&useSSL=false&zeroDateTimeBehavior=convertToNull&serverTimezone=Asia/Shanghai");
        entDsParams.put(DBCPDataSourceFactory.USERNAME.key, enterpriseConfig.getDsUserName());
        entDsParams.put(DBCPDataSourceFactory.PASSWORD.key, enterpriseConfig.getDsPassword());
        entDsParams.put(DBCPDataSourceFactory.MAXACTIVE.key, Integer.valueOf(10));
        entDsParams.put(DBCPDataSourceFactory.MAXIDLE.key, Integer.valueOf(1));
        entDsParams.put(JDBCDataStoreFactory.HOST.key, enterpriseConfig.getDsIp());
        entDsParams.put(JDBCDataStoreFactory.DATABASE.key, enterpriseConfig.getDsBaseName());
        entDsParams.put(JDBCDataStoreFactory.PORT.key, enterpriseConfig.getDsPort());
        entDsParams.put("tableName", enterpriseConfig.getEnterprise());

        //组转cql查询条件
        StringBuffer filterCql = new StringBuffer();
        if(enterpriseVo.getEntName() !=null && !"".equals(enterpriseVo.getEntName())){

            filterCql.append("(qymc like '%"+enterpriseVo.getEntName()+"%')");

        }

        if(enterpriseVo.getEntType() !=null && !"".equals(enterpriseVo.getEntType())){

            String result = transEntTypeListToSqlString(Arrays.asList(enterpriseVo.getEntType().split(",")));

            if(filterCql.length() >0){

                filterCql.append(" and (qykx in ("+result+"))");
            }else{

                filterCql.append(" (qykx in ("+result+"))");
            }

        }

        Map<String, Object> mapParams = new HashMap<String, Object>();

        if(filterCql.length() >0){

            mapParams.put("cql", filterCql.toString());
        }

        //组装分页参数
        mapParams.put("pageSize",enterpriseVo.getPageSize());
        mapParams.put("pageNumber",enterpriseVo.getPageNumber());

        Map<String, Object> sortMaps = new HashMap<String, Object>();
        sortMaps.put("qymc","asc");
        mapParams.put("sort",JSONObject.toJSONString(sortMaps));

        System.out.println("查询条件:"+filterCql.toString());

        PageResult pageResult = ecqlService.cqlQuery(entDsParams,mapParams);

        List<EnterpriseInfo> enterpriseInfoList = new ArrayList<EnterpriseInfo>();

        if(pageResult.getResp_code() ==0) {

            List<Map<String, Object>> dataList = (List<Map<String, Object>>) pageResult.getDatas();

            for (Map<String, Object> dataMap : dataList) {

                if (dataMap.containsKey("location")) {

                    dataMap.remove("location");

                }

                EnterpriseInfo enterpriseInfo = JSONObject.parseObject(JSONObject.toJSONString(dataMap), EnterpriseInfo.class);
                enterpriseInfo.setDataId("qy_"+enterpriseInfo.getId());

                enterpriseInfoList.add(enterpriseInfo);
            }
        }

        pageResult.setDatas(enterpriseInfoList);



        return pageResult;
    }

    public String transEntTypeListToSqlString(List<String> entTypeList) {

        StringBuilder sqlString = new StringBuilder();

        for(String item:entTypeList){

            sqlString.append("'"+item+"',");

        }

        String result = sqlString.toString();

        return result.substring(0,result.lastIndexOf(","));
    }

    @Override
    public Result deviceQuery(DeviceVo deviceVo) {

        //构建mysql数据库连接数据源--DataSource
        Map<String, Object> mapParams = new HashMap<String, Object>();
        mapParams.put(DBCPDataSourceFactory.DSTYPE.key, "DBCP");
        mapParams.put(DBCPDataSourceFactory.DRIVERCLASS.key, "com.mysql.cj.jdbc.Driver");
        mapParams.put(DBCPDataSourceFactory.JDBC_URL.key, "jdbc:mysql://"+enterpriseConfig.getDsIp()+":"+enterpriseConfig.getDsPort()+"/"+enterpriseConfig.getDsBaseName()+"?useUnicode=true&characterEncoding=UTF-8&autoReconnect=true&useSSL=false&zeroDateTimeBehavior=convertToNull&serverTimezone=Asia/Shanghai");
        mapParams.put(DBCPDataSourceFactory.USERNAME.key, enterpriseConfig.getDsUserName());
        mapParams.put(DBCPDataSourceFactory.PASSWORD.key, enterpriseConfig.getDsPassword());
        mapParams.put(DBCPDataSourceFactory.MAXACTIVE.key, Integer.valueOf(10));
        mapParams.put(DBCPDataSourceFactory.MAXIDLE.key, Integer.valueOf(1));
        mapParams.put(JDBCDataStoreFactory.HOST.key, enterpriseConfig.getDsIp());
        mapParams.put(JDBCDataStoreFactory.DATABASE.key, enterpriseConfig.getDsBaseName());


        String sql = "SELECT *,(6371 * acos (cos ( radians("+deviceVo.getLat()+") )* cos( radians(latitude) )* cos( radians(longitude) - radians("+deviceVo.getLon()+") )+ sin ( radians("+deviceVo.getLat()+"))* sin( radians(latitude) ))) AS distance FROM "+enterpriseConfig.getDevice()+" having distance <"+deviceVo.getRadius()+" order by distance asc";

        PageResult pageResult = sqlService.sqlQuery4Jdbc(mapParams,sql,null);

        List<DeviceInfo> deviceInfoList = new ArrayList<DeviceInfo>();

        if(pageResult.getResp_code() ==0) {

            List<Map<String, Object>> dataList = (List<Map<String, Object>>) pageResult.getDatas();

            for (Map<String, Object> dataMap : dataList) {

                if (dataMap.containsKey("location")) {

                    dataMap.remove("location");

                }

                DeviceInfo deviceInfo = JSONObject.parseObject(JSONObject.toJSONString(dataMap), DeviceInfo.class);
                deviceInfo.setDataId("sbzy_"+deviceInfo.getId());

                deviceInfoList.add(deviceInfo);
            }
        }

        return Result.of(deviceInfoList,pageResult.getResp_code(),pageResult.getResp_msg());


    }


    public Result deviceQuery2(DeviceVo deviceVo) {

        //数据源
        Map<String, Object> entDsParams = new HashMap<String, Object>();
        entDsParams.put(DBCPDataSourceFactory.DSTYPE.key, "DBCP");
        entDsParams.put(DBCPDataSourceFactory.DRIVERCLASS.key, "com.mysql.cj.jdbc.Driver");
        entDsParams.put(DBCPDataSourceFactory.JDBC_URL.key, "jdbc:mysql://"+enterpriseConfig.getDsIp()+":"+enterpriseConfig.getDsPort()+"/"+enterpriseConfig.getDsBaseName()+"?useUnicode=true&characterEncoding=UTF-8&autoReconnect=true&useSSL=false&zeroDateTimeBehavior=convertToNull&serverTimezone=Asia/Shanghai");
        entDsParams.put(DBCPDataSourceFactory.USERNAME.key, enterpriseConfig.getDsUserName());
        entDsParams.put(DBCPDataSourceFactory.PASSWORD.key, enterpriseConfig.getDsPassword());
        entDsParams.put(DBCPDataSourceFactory.MAXACTIVE.key, Integer.valueOf(10));
        entDsParams.put(DBCPDataSourceFactory.MAXIDLE.key, Integer.valueOf(1));
        entDsParams.put(JDBCDataStoreFactory.HOST.key, enterpriseConfig.getDsIp());
        entDsParams.put(JDBCDataStoreFactory.DATABASE.key, enterpriseConfig.getDsBaseName());
        entDsParams.put(JDBCDataStoreFactory.PORT.key, enterpriseConfig.getDsPort());
        entDsParams.put("tableName", enterpriseConfig.getDevice());

        //组转cql查询条件
        StringBuffer filterCql = new StringBuffer();

        String wkt = "'POINT("+deviceVo.getLon()+" "+deviceVo.getLat()+")'";

        filterCql.append("(DWITHIN(location," + wkt + ", "+deviceVo.getRadius()+", kilometers))");

        Map<String, Object> mapParams = new HashMap<String, Object>();

        if(filterCql.length() >0){

            mapParams.put("cql", filterCql.toString());
        }



        PageResult pageResult = ecqlService.cqlQuery(entDsParams,mapParams);

        List<DeviceInfo> deviceInfoList = new ArrayList<DeviceInfo>();

        if(pageResult.getResp_code() ==0){

            List<Map<String, Object>> dataList = (List<Map<String, Object>>) pageResult.getDatas();

            for(Map<String, Object> dataMap:dataList){

                if(dataMap.containsKey("location")){

                    dataMap.remove("location");

                }



                try {

                    DeviceInfo deviceInfo = JSONObject.parseObject(JSONObject.toJSONString(dataMap), DeviceInfo.class);

                    CoordinateReferenceSystem crs = DefaultGeographicCRS.WGS84;

                    Coordinate c1 = new Coordinate(Double.parseDouble(deviceInfo.getLongitude()), Double.parseDouble(deviceInfo.getLatitude()));
                    Coordinate c2 = new Coordinate(deviceVo.getLon(), deviceVo.getLat());

                    double dist = JTS.orthodromicDistance(c1, c2, crs);

                    if (dist <= deviceVo.getRadius()*1000){

                        deviceInfo.setDistance(dist);

                        deviceInfoList.add(deviceInfo);
                    }



                }catch (Exception exception){
                    exception.printStackTrace();
                }


            }
        }

        Collections.sort(deviceInfoList, new DeviceComparator());

        System.out.println("查询条件:"+filterCql.toString()+" 记录数:"+deviceInfoList.size());


        return Result.of(deviceInfoList,pageResult.getResp_code(),pageResult.getResp_msg());


    }

    @Override
    public Result staticsQuery() {

        //数据源

        Map<String, Object> entDsParams = new HashMap<String, Object>();
        entDsParams.put(DBCPDataSourceFactory.DSTYPE.key, "DBCP");
        entDsParams.put(DBCPDataSourceFactory.DRIVERCLASS.key, "com.mysql.cj.jdbc.Driver");
        entDsParams.put(DBCPDataSourceFactory.JDBC_URL.key, "jdbc:mysql://"+enterpriseConfig.getDsIp()+":"+enterpriseConfig.getDsPort()+"/"+enterpriseConfig.getDsBaseName()+"?useUnicode=true&characterEncoding=UTF-8&autoReconnect=true&useSSL=false&zeroDateTimeBehavior=convertToNull&serverTimezone=Asia/Shanghai");
        entDsParams.put(DBCPDataSourceFactory.USERNAME.key, enterpriseConfig.getDsUserName());
        entDsParams.put(DBCPDataSourceFactory.PASSWORD.key, enterpriseConfig.getDsPassword());
        entDsParams.put(DBCPDataSourceFactory.MAXACTIVE.key, Integer.valueOf(10));
        entDsParams.put(DBCPDataSourceFactory.MAXIDLE.key, Integer.valueOf(1));
        entDsParams.put(JDBCDataStoreFactory.HOST.key, enterpriseConfig.getDsIp());
        entDsParams.put(JDBCDataStoreFactory.DATABASE.key, enterpriseConfig.getDsBaseName());
        entDsParams.put(JDBCDataStoreFactory.PORT.key, enterpriseConfig.getDsPort());
        entDsParams.put("tableName", enterpriseConfig.getEnterprise());

        //计算企业总数

        CountVisitor countVisitor = new CountVisitor();
        ecqlService.cqlAggQuery(entDsParams,null,countVisitor);

        //计算零关税自用进口生产设备数量
        CountVisitor deviceCountVisitor = new CountVisitor();
        String cqlDevice = "(qykx ='零关税自用进口生产设备')";
        ecqlService.cqlAggQuery(entDsParams,cqlDevice,deviceCountVisitor);

        //计算零关税进口原辅料数量
        CountVisitor materialCountVisitor = new CountVisitor();
        String cqlMaterial = "(qykx ='零关税进口原辅料')";
        ecqlService.cqlAggQuery(entDsParams,cqlMaterial,materialCountVisitor);

        //计算加工增值免关税数量
        CountVisitor noTaxCountVisitor = new CountVisitor();
        String cqlNoTax = "(qykx ='加工增值免关税')";
        ecqlService.cqlAggQuery(entDsParams,cqlNoTax,noTaxCountVisitor);

        StaticsInfo staticsInfo = new StaticsInfo();
        staticsInfo.setCount(countVisitor.getCount());
        staticsInfo.setDeviceCount(deviceCountVisitor.getCount());
        staticsInfo.setMaterialCount(materialCountVisitor.getCount());
        staticsInfo.setNoTaxCount(noTaxCountVisitor.getCount());

        if(countVisitor.getCount()>0){

            BigDecimal deviceRate = new BigDecimal(deviceCountVisitor.getCount()).divide(new BigDecimal(countVisitor.getCount()),3,RoundingMode.HALF_UP).multiply(new BigDecimal(100)).setScale(2, RoundingMode.HALF_UP);

            staticsInfo.setDeviceRate(deviceRate.stripTrailingZeros());

            BigDecimal materialRate = new BigDecimal(materialCountVisitor.getCount()).divide(new BigDecimal(countVisitor.getCount()),3,RoundingMode.HALF_UP).multiply(new BigDecimal(100)).setScale(2, RoundingMode.HALF_UP);
            staticsInfo.setMaterialRate(materialRate.stripTrailingZeros());

            BigDecimal noTaxRate = new BigDecimal(noTaxCountVisitor.getCount()).divide(new BigDecimal(countVisitor.getCount()),3,RoundingMode.HALF_UP).multiply(new BigDecimal(100)).setScale(2, RoundingMode.HALF_UP);

            staticsInfo.setNoTaxRate(noTaxRate.stripTrailingZeros());

        }


        return Result.of(staticsInfo,0,"操作成功！");
    }

    //设备距离排序器
    class  DeviceComparator implements Comparator<DeviceInfo>{

        @Override
        public int compare(DeviceInfo o1, DeviceInfo o2) {

            return o1.getDistance().compareTo(o2.getDistance());
        }
    }

    //企业名称顺序排序器
    class  EntNameComparator implements Comparator<EnterpriseInfo>{

        @Override
        public int compare(EnterpriseInfo o1, EnterpriseInfo o2) {

            return o1.getQymc().compareTo(o2.getQymc());
        }
    }


}
