package cn.com.taiji.web.service.impl;

import cn.com.taiji.common.domain.PageResult;
import cn.com.taiji.common.domain.Result;
import cn.com.taiji.common.model.LayerGroupDsView;
import cn.com.taiji.common.model.LayerStyleView;
import cn.com.taiji.cql.service.IECqlService;
import cn.com.taiji.tile.util.StringUtils;
import cn.com.taiji.web.config.GeoConfig;
import cn.com.taiji.web.service.IGeoDataSourceService;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.geotools.data.jdbc.datasource.DBCPDataSourceFactory;
import org.geotools.data.jdbc.datasource.DataSourceFinder;
import org.geotools.jdbc.JDBCDataStoreFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.sql.DataSource;
import java.util.*;

@Slf4j
@Service
public class GeoDataSourceServiceImpl implements IGeoDataSourceService {

    @Autowired
    private GeoConfig geoConfig;

    @Autowired
    private IECqlService ecqlService;


    @Override
    public List<LayerGroupDsView> layerCodeGeoDataSource(List<String> layerCodeList) throws Exception{


        List<LayerGroupDsView> layerGroupDsViewList = new ArrayList<LayerGroupDsView>();

        Map<String, Object> dsParams = new HashMap<String, Object>();
        dsParams.put(DBCPDataSourceFactory.DSTYPE.key, "DBCP");
        dsParams.put(DBCPDataSourceFactory.DRIVERCLASS.key, "com.mysql.cj.jdbc.Driver");
        dsParams.put(DBCPDataSourceFactory.JDBC_URL.key, "jdbc:mysql://"+geoConfig.getDsIp()+":"+geoConfig.getDsPort()+"/"+geoConfig.getDsBase()+"?useUnicode=true&characterEncoding=UTF-8&autoReconnect=true&useSSL=false&zeroDateTimeBehavior=convertToNull&serverTimezone=Asia/Shanghai");
        dsParams.put(DBCPDataSourceFactory.USERNAME.key, geoConfig.getDsUsername());
        dsParams.put(DBCPDataSourceFactory.PASSWORD.key, geoConfig.getDsPassword());
        dsParams.put(DBCPDataSourceFactory.MAXACTIVE.key, Integer.valueOf(10));
        dsParams.put(DBCPDataSourceFactory.MAXIDLE.key, Integer.valueOf(1));
        dsParams.put(JDBCDataStoreFactory.HOST.key, geoConfig.getDsIp());
        dsParams.put(JDBCDataStoreFactory.DATABASE.key, geoConfig.getDsBase());
        dsParams.put(JDBCDataStoreFactory.PORT.key, geoConfig.getDsPort());
        dsParams.put("tableName", "layer_group_ds_view");


        String layerCodeCql = "(group_code IN ("+StringUtils.convertListToString(layerCodeList)+") or layer_code in ("+StringUtils.convertListToString(layerCodeList)+"))";

        Map<String, Object> mapParams = new HashMap<String, Object>();
        mapParams.put("cql", layerCodeCql);

        Map<String, Object> sortMaps = new HashMap<String, Object>();
        sortMaps.put("sort","asc");
        mapParams.put("sort",JSONObject.toJSONString(sortMaps));

        PageResult pageResult = ecqlService.cqlQuery(dsParams,mapParams);

        if(pageResult.getRecordCount() >0){

            List<Map<String, Object>> dataSourceList = (List<Map<String, Object>>) pageResult.getDatas();

            layerGroupDsViewList = JSONObject.parseArray(JSONObject.toJSONString(dataSourceList), LayerGroupDsView.class);

        }

        return layerGroupDsViewList;



    }

    @Override
    public Map<String,List<LayerStyleView>> layerCodeShapeStyle(List<String> layerCodeList) throws Exception {

        Map<String,List<LayerStyleView>> dataMaps = new HashMap<String,List<LayerStyleView>>();
        if(CollUtil.isEmpty(layerCodeList)){
            return dataMaps;
        }

        Map<String, Object> dsParams = new HashMap<String, Object>();
        dsParams.put(DBCPDataSourceFactory.DSTYPE.key, "DBCP");
        dsParams.put(DBCPDataSourceFactory.DRIVERCLASS.key, "com.mysql.cj.jdbc.Driver");
        dsParams.put(DBCPDataSourceFactory.JDBC_URL.key, "jdbc:mysql://"+geoConfig.getDsIp()+":"+geoConfig.getDsPort()+"/"+geoConfig.getDsBase()+"?useUnicode=true&characterEncoding=UTF-8&autoReconnect=true&useSSL=false&zeroDateTimeBehavior=convertToNull&serverTimezone=Asia/Shanghai");
        dsParams.put(DBCPDataSourceFactory.USERNAME.key, geoConfig.getDsUsername());
        dsParams.put(DBCPDataSourceFactory.PASSWORD.key, geoConfig.getDsPassword());
        dsParams.put(DBCPDataSourceFactory.MAXACTIVE.key, Integer.valueOf(10));
        dsParams.put(DBCPDataSourceFactory.MAXIDLE.key, Integer.valueOf(1));
        dsParams.put(JDBCDataStoreFactory.HOST.key, geoConfig.getDsIp());
        dsParams.put(JDBCDataStoreFactory.DATABASE.key, geoConfig.getDsBase());
        dsParams.put(JDBCDataStoreFactory.PORT.key, geoConfig.getDsPort());
        dsParams.put("tableName", "layer_style_view");


        String layerCodeCql = "layer_code IN (" + StringUtils.convertListToString(layerCodeList) + ")";

        Map<String, Object> mapParams = new HashMap<String, Object>();
        mapParams.put("cql", layerCodeCql);

        Map<String, Object> sortMaps = new HashMap<String, Object>();
        sortMaps.put("sort","asc");
        mapParams.put("sort",JSONObject.toJSONString(sortMaps));

        Map<String, Object> sortParams = new HashMap<String, Object>();

        PageResult pageResult = ecqlService.cqlQuery(dsParams,mapParams);

        if(pageResult.getRecordCount() >0){

            List<Map<String, Object>> dataSourceList = (List<Map<String, Object>>) pageResult.getDatas();

            List<LayerStyleView> layerStyleViewList = JSONObject.parseArray(JSONObject.toJSONString(dataSourceList), LayerStyleView.class);

            for(LayerStyleView lsv:layerStyleViewList){

                String layerCode = lsv.getLayerCode();

                if(dataMaps.containsKey(layerCode)){

                    List<LayerStyleView> tempList = dataMaps.get(layerCode);
                    tempList.add(lsv);

                }else{

                    List<LayerStyleView> newList = new ArrayList<LayerStyleView>();
                    newList.add(lsv);

                    dataMaps.put(layerCode,newList);

                }

            }



        }

        return dataMaps;
    }

    public DataSource defaultDataSource() throws Exception{

        //构建mysql数据库连接数据源--DataSource
        Map<String, Object> mapParams = new HashMap<String, Object>();
        mapParams.put(DBCPDataSourceFactory.DSTYPE.key, "DBCP");
        mapParams.put(DBCPDataSourceFactory.DRIVERCLASS.key, "com.mysql.cj.jdbc.Driver");
        mapParams.put(DBCPDataSourceFactory.JDBC_URL.key, "jdbc:mysql://"+geoConfig.getDsIp()+":"+geoConfig.getDsPort()+"/"+geoConfig.getDsBase()+"?useUnicode=true&characterEncoding=UTF-8&autoReconnect=true&useSSL=false&zeroDateTimeBehavior=convertToNull&serverTimezone=Asia/Shanghai");
        mapParams.put(DBCPDataSourceFactory.USERNAME.key, geoConfig.getDsUsername());
        mapParams.put(DBCPDataSourceFactory.PASSWORD.key, geoConfig.getDsPassword());
        mapParams.put(DBCPDataSourceFactory.MAXACTIVE.key, Integer.valueOf(10));
        mapParams.put(DBCPDataSourceFactory.MAXIDLE.key, Integer.valueOf(0));

        return DataSourceFinder.getDataSource(mapParams);

    }

    @Override
    public PageResult geoDsQuery(Map<String, Object> paramMaps) throws Exception {

        Map<String, Object> dsParams = new HashMap<String, Object>();
        dsParams.put(DBCPDataSourceFactory.DSTYPE.key, "DBCP");
        dsParams.put(DBCPDataSourceFactory.DRIVERCLASS.key, "com.mysql.cj.jdbc.Driver");
        dsParams.put(DBCPDataSourceFactory.JDBC_URL.key, "jdbc:mysql://"+geoConfig.getDsIp()+":"+geoConfig.getDsPort()+"/"+geoConfig.getDsBase()+"?useUnicode=true&characterEncoding=UTF-8&autoReconnect=true&useSSL=false&zeroDateTimeBehavior=convertToNull&serverTimezone=Asia/Shanghai");
        dsParams.put(DBCPDataSourceFactory.USERNAME.key, geoConfig.getDsUsername());
        dsParams.put(DBCPDataSourceFactory.PASSWORD.key, geoConfig.getDsPassword());
        dsParams.put(DBCPDataSourceFactory.MAXACTIVE.key, Integer.valueOf(10));
        dsParams.put(DBCPDataSourceFactory.MAXIDLE.key, Integer.valueOf(1));
        dsParams.put(JDBCDataStoreFactory.HOST.key, geoConfig.getDsIp());
        dsParams.put(JDBCDataStoreFactory.DATABASE.key, geoConfig.getDsBase());
        dsParams.put(JDBCDataStoreFactory.PORT.key, geoConfig.getDsPort());
        dsParams.put("tableName", "tile_data_source");

        return ecqlService.cqlQuery(dsParams,paramMaps);
    }

    @Override
    public Result geoDsAddInfo(Map<String, Object> paramMaps) throws Exception {

        Map<String, Object> dsParams = new HashMap<String, Object>();
        dsParams.put(DBCPDataSourceFactory.DSTYPE.key, "DBCP");
        dsParams.put(DBCPDataSourceFactory.DRIVERCLASS.key, "com.mysql.cj.jdbc.Driver");
        dsParams.put(DBCPDataSourceFactory.JDBC_URL.key, "jdbc:mysql://"+geoConfig.getDsIp()+":"+geoConfig.getDsPort()+"/"+geoConfig.getDsBase()+"?useUnicode=true&characterEncoding=UTF-8&autoReconnect=true&useSSL=false&zeroDateTimeBehavior=convertToNull&serverTimezone=Asia/Shanghai");
        dsParams.put(DBCPDataSourceFactory.USERNAME.key, geoConfig.getDsUsername());
        dsParams.put(DBCPDataSourceFactory.PASSWORD.key, geoConfig.getDsPassword());
        dsParams.put(DBCPDataSourceFactory.MAXACTIVE.key, Integer.valueOf(10));
        dsParams.put(DBCPDataSourceFactory.MAXIDLE.key, Integer.valueOf(1));
        dsParams.put(JDBCDataStoreFactory.HOST.key, geoConfig.getDsIp());
        dsParams.put(JDBCDataStoreFactory.DATABASE.key, geoConfig.getDsBase());
        dsParams.put(JDBCDataStoreFactory.PORT.key, geoConfig.getDsPort());
        dsParams.put("tableName", "tile_data_source");

        DateTime beginTime = DateUtil.parse(DateUtil.formatDateTime(new Date()), DatePattern.NORM_DATETIME_PATTERN);

        paramMaps.put("create_time",beginTime);

        return ecqlService.cqlAdd(dsParams,paramMaps);
    }

    @Override
    public Result geoDsUpdateInfo(Map<String, Object> paramMaps) throws Exception {

        Map<String, Object> dsParams = new HashMap<String, Object>();
        dsParams.put(DBCPDataSourceFactory.DSTYPE.key, "DBCP");
        dsParams.put(DBCPDataSourceFactory.DRIVERCLASS.key, "com.mysql.cj.jdbc.Driver");
        dsParams.put(DBCPDataSourceFactory.JDBC_URL.key, "jdbc:mysql://"+geoConfig.getDsIp()+":"+geoConfig.getDsPort()+"/"+geoConfig.getDsBase()+"?useUnicode=true&characterEncoding=UTF-8&autoReconnect=true&useSSL=false&zeroDateTimeBehavior=convertToNull&serverTimezone=Asia/Shanghai");
        dsParams.put(DBCPDataSourceFactory.USERNAME.key, geoConfig.getDsUsername());
        dsParams.put(DBCPDataSourceFactory.PASSWORD.key, geoConfig.getDsPassword());
        dsParams.put(DBCPDataSourceFactory.MAXACTIVE.key, Integer.valueOf(10));
        dsParams.put(DBCPDataSourceFactory.MAXIDLE.key, Integer.valueOf(1));
        dsParams.put(JDBCDataStoreFactory.HOST.key, geoConfig.getDsIp());
        dsParams.put(JDBCDataStoreFactory.DATABASE.key, geoConfig.getDsBase());
        dsParams.put(JDBCDataStoreFactory.PORT.key, geoConfig.getDsPort());
        dsParams.put("tableName", "tile_data_source");

        DateTime updateTime = DateUtil.parse(DateUtil.formatDateTime(new Date()), DatePattern.NORM_DATETIME_PATTERN);

        paramMaps.put("update_time",updateTime);

        String cql = paramMaps.get("cql").toString();

        if(paramMaps.containsKey("cql")){

            paramMaps.remove("cql");
        }

        return ecqlService.cqlUpdate(dsParams,paramMaps,cql);
    }

    @Override
    public Result geoDsRemoveInfo(Map<String, Object> paramMaps) throws Exception {

        Map<String, Object> dsParams = new HashMap<String, Object>();
        dsParams.put(DBCPDataSourceFactory.DSTYPE.key, "DBCP");
        dsParams.put(DBCPDataSourceFactory.DRIVERCLASS.key, "com.mysql.cj.jdbc.Driver");
        dsParams.put(DBCPDataSourceFactory.JDBC_URL.key, "jdbc:mysql://"+geoConfig.getDsIp()+":"+geoConfig.getDsPort()+"/"+geoConfig.getDsBase()+"?useUnicode=true&characterEncoding=UTF-8&autoReconnect=true&useSSL=false&zeroDateTimeBehavior=convertToNull&serverTimezone=Asia/Shanghai");
        dsParams.put(DBCPDataSourceFactory.USERNAME.key, geoConfig.getDsUsername());
        dsParams.put(DBCPDataSourceFactory.PASSWORD.key, geoConfig.getDsPassword());
        dsParams.put(DBCPDataSourceFactory.MAXACTIVE.key, Integer.valueOf(10));
        dsParams.put(DBCPDataSourceFactory.MAXIDLE.key, Integer.valueOf(1));
        dsParams.put(JDBCDataStoreFactory.HOST.key, geoConfig.getDsIp());
        dsParams.put(JDBCDataStoreFactory.DATABASE.key, geoConfig.getDsBase());
        dsParams.put(JDBCDataStoreFactory.PORT.key, geoConfig.getDsPort());
        dsParams.put("tableName", "tile_data_source");

        String cql = paramMaps.get("cql").toString();

        return ecqlService.cqlRemove(dsParams,cql);
    }
}
